""" Copyright 2012, 2013 UW Information Technology, University of Washington

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from django.test import TestCase
from django.conf import settings
from django.core.management import call_command
from spotseeker_server.models import Spot, TrustedOAuthClient
import simplejson as json
import hashlib
import time
import random
from oauth_provider.models import Consumer
import oauth2
from django.test.utils import override_settings
from mock import patch
from spotseeker_server import models


@override_settings(SPOTSEEKER_AUTH_MODULE='spotseeker_server.auth.oauth')
class SpotAuthOAuth(TestCase):

    def setUp(self):
        spot = Spot.objects.create(
            name="This is for testing the oauth module",
            capacity=10
        )
        self.spot = spot
        self.url = "/api/v1/spot/%s" % self.spot.pk

    def test_get_no_oauth(self):
        c = self.client
        response = c.get(self.url)
        self.assertEqual(response.status_code, 401,
                          "No access to GET w/o oauth")

    def test_valid_oauth(self):
        consumer_name = "Test consumer"

        key = hashlib.sha1("{0} - {1}".format(random.random(),
                                              time.time()
                                              )
                           ).hexdigest()
        secret = hashlib.sha1("{0} - {1}".format(random.random(),
                                                 time.time()
                                                 )
                              ).hexdigest()

        create_consumer = Consumer.objects.create(
            name=consumer_name,
            key=key,
            secret=secret
        )

        consumer = oauth2.Consumer(key=key, secret=secret)

        req = oauth2.Request.from_consumer_and_token(
            consumer,
            None,
            http_method='GET',
            http_url="http://testserver/api/v1/spot/%s" % self.spot.pk
        )

        oauth_header = req.to_header()

        c = self.client
        response = c.get(
            self.url,
            HTTP_AUTHORIZATION=oauth_header['Authorization']
        )

        self.assertEqual(
            response.status_code,
            200,
            "Got a 200 w/ a proper oauth client connection"
        )

        spot_dict = json.loads(response.content)

        self.assertEqual(
            spot_dict['id'],
            self.spot.pk,
            "Got the right spot back from oauth"
        )

    def test_invalid_oauth(self):
        consumer = oauth2.Consumer(
            key="This is a fake key",
            secret="This is a fake secret"
        )

        req = oauth2.Request.from_consumer_and_token(
            consumer,
            None,
            http_method='GET',
            http_url="http://testserver/api/v1/spot/%s" % self.spot.pk
        )

        oauth_header = req.to_header()

        c = self.client
        response = c.get(self.url,
                         HTTP_AUTHORIZATION=oauth_header['Authorization'])

        self.assertEqual(
            response.status_code,
            401,
            "Got a 401 w/ an invented oauth client id"
        )

    @override_settings(
        SPOTSEEKER_SPOT_FORM='spotseeker_server.default_forms.sp'
        'ot.DefaultSpotForm'
    )
    @override_settings(
        SPOTSEEKER_SPOTEXTENDEDINFO_FORM='spotseeker_server.default_forms.sp'
        'ot.DefaultSpotExtendedInfoForm'
    )
    def test_put_no_oauth(self):
        c = self.client

        response = c.get(self.url)

        etag = self.spot.etag

        spot_dict = self.spot.json_data_structure()
        spot_dict['name'] = "Failing to modify oauth"

        response = c.put(self.url, json.dumps(spot_dict),
                         content_type="application/json", If_Match=etag)
        self.assertEqual(response.status_code, 401,
                          "Rejects a PUT w/o oauth info")

    @override_settings(
        SPOTSEEKER_SPOT_FORM='spotseeker_server.default_forms.sp'
        'ot.DefaultSpotForm'
    )
    @override_settings(
        SPOTSEEKER_SPOTEXTENDEDINFO_FORM='spotseeker_server.default_forms.sp'
        'ot.DefaultSpotExtendedInfoForm'
    )
    def test_put_untrusted_oauth(self):
        consumer_name = "Untrusted test consumer"

        key = hashlib.sha1("{0} - {1}".format(random.random(),
                                              time.time()
                                              )
                           ).hexdigest()
        secret = hashlib.sha1("{0} - {1}".format(random.random(),
                                                 time.time()
                                                 )
                              ).hexdigest()

        create_consumer = Consumer.objects.create(
            name=consumer_name,
            key=key,
            secret=secret
        )

        consumer = oauth2.Consumer(key=key, secret=secret)

        req = oauth2.Request.from_consumer_and_token(
            consumer,
            None,
            http_method='GET',
            http_url="http://testserver/api/v1/spot/%s" % self.spot.pk
        )

        oauth_header = req.to_header()
        c = self.client

        response = c.get(
            self.url,
            HTTP_AUTHORIZATION=oauth_header['Authorization']
        )
        etag = response["ETag"]

        spot_dict = json.loads(response.content)
        spot_dict['name'] = "Failing to modify oauth"
        spot_dict['location'] = {"latitude": 55, "longitude": -30}

        response = c.put(
            self.url,
            json.dumps(spot_dict),
            content_type="application/json",
            If_Match=etag,
            HTTP_AUTHORIZATION=oauth_header['Authorization']
        )
        self.assertEqual(
            response.status_code,
            401,
            "Rejects a PUT from a non-trusted oauth client"
        )

    @override_settings(
        SPOTSEEKER_SPOT_FORM='spotseeker_server.default_forms.sp'
        'ot.DefaultSpotForm'
    )
    @override_settings(
        SPOTSEEKER_SPOTEXTENDEDINFO_FORM='spotseeker_server.default_forms.sp'
        'ot.DefaultSpotExtendedInfoForm'
    )
    def test_put_untrusted_oauth_with_user_header(self):
        consumer_name = "Untrusted test consumer"

        key = hashlib.sha1("{0} - {1}".format(random.random(),
                                              time.time()
                                              )
                           ).hexdigest()
        secret = hashlib.sha1("{0} - {1}".format(random.random(),
                                                 time.time()
                                                 )
                              ).hexdigest()

        create_consumer = Consumer.objects.create(
            name=consumer_name,
            key=key,
            secret=secret
        )

        consumer = oauth2.Consumer(key=key, secret=secret)

        req = oauth2.Request.from_consumer_and_token(
            consumer,
            None,
            http_method='GET',
            http_url="http://testserver/api/v1/spot/%s" % self.spot.pk
        )

        oauth_header = req.to_header()
        c = self.client

        response = c.get(
            self.url,
            HTTP_AUTHORIZATION=oauth_header['Authorization']
        )
        etag = response["ETag"]

        spot_dict = json.loads(response.content)
        spot_dict['name'] = "Failing to modify oauth"

        response = c.put(
            self.url,
            json.dumps(spot_dict),
            content_type="application/json",
            If_Match=etag,
            HTTP_AUTHORIZATION=oauth_header['Authorization'],
            HTTP_X_OAUTH_USER="pmichaud"
        )
        self.assertEqual(
            response.status_code,
            401,
            "Rejects a PUT from a non-trusted oauth client"
        )

    @override_settings(
        SPOTSEEKER_SPOT_FORM='spotseeker_server.default_forms.sp'
        'ot.DefaultSpotForm'
    )
    @override_settings(
        SPOTSEEKER_SPOTEXTENDEDINFO_FORM='spotseeker_server.default_forms.sp'
        'ot.DefaultSpotExtendedInfoForm'
    )
    @override_settings(SPOTSEEKER_AUTH_ADMINS=('pmichaud',))
    def test_put_trusted_client(self):
        consumer_name = "Trusted test consumer"

        key = hashlib.sha1("{0} - {1}".format(random.random(),
                                              time.time()
                                              )
                           ).hexdigest()
        secret = hashlib.sha1("{0} - {1}".format(random.random(),
                                                 time.time()
                                                 )
                              ).hexdigest()

        create_consumer = Consumer.objects.create(name=consumer_name,
                                                  key=key,
                                                  secret=secret)
        trusted_consumer = TrustedOAuthClient.objects.create(
            consumer=create_consumer,
            is_trusted=True,
            bypasses_user_authorization=False
        )

        consumer = oauth2.Consumer(key=key, secret=secret)

        req = oauth2.Request.from_consumer_and_token(
            consumer,
            None,
            http_method='GET',
            http_url="http://testserver/api/v1/spot/%s" % self.spot.pk
        )

        oauth_header = req.to_header()
        c = self.client

        response = c.get(self.url,
                         HTTP_AUTHORIZATION=oauth_header['Authorization'])
        etag = response["ETag"]

        spot_dict = json.loads(response.content)
        spot_dict['name'] = "Failing to modify oauth"
        spot_dict['location'] = {"latitude": 55, "longitude": -30}

        response = c.put(
            self.url,
            json.dumps(spot_dict),
            content_type="application/json",
            If_Match=etag,
            HTTP_AUTHORIZATION=oauth_header['Authorization'],
            HTTP_X_OAUTH_USER="pmichaud"
        )
        self.assertEqual(
            response.status_code,
            200,
            "Accepts a PUT from a trusted oauth client"
        )

    @override_settings(
        SPOTSEEKER_SPOT_FORM='spotseeker_server.default_forms.sp'
        'ot.DefaultSpotForm'
    )
    @override_settings(
        SPOTSEEKER_SPOTEXTENDEDINFO_FORM='spotseeker_server.default_forms.sp'
        'ot.DefaultSpotExtendedInfoForm'
    )
    def test_put_trusted_client_no_user(self):
        consumer_name = "Trusted test consumer"

        key = hashlib.sha1("{0} - {1}".format(random.random(),
                                              time.time()
                                              )
                           ).hexdigest()
        secret = hashlib.sha1("{0} - {1}".format(random.random(),
                                                 time.time()
                                                 )
                              ).hexdigest()

        create_consumer = Consumer.objects.create(name=consumer_name,
                                                  key=key,
                                                  secret=secret)
        trusted_consumer = TrustedOAuthClient.objects.create(
            consumer=create_consumer,
            is_trusted=True,
            bypasses_user_authorization=False
        )

        consumer = oauth2.Consumer(key=key, secret=secret)

        req = oauth2.Request.from_consumer_and_token(
            consumer,
            None,
            http_method='GET',
            http_url="http://testserver/api/v1/spot/%s" % self.spot.pk
        )

        oauth_header = req.to_header()
        c = self.client

        response = c.get(
            self.url,
            HTTP_AUTHORIZATION=oauth_header['Authorization']
        )
        etag = response["ETag"]

        spot_dict = json.loads(response.content)
        spot_dict['name'] = "Failing to modify oauth"

        response = c.put(
            self.url,
            json.dumps(spot_dict),
            content_type="application/json",
            If_Match=etag,
            HTTP_AUTHORIZATION=oauth_header['Authorization']
        )
        self.assertEqual(
            response.status_code,
            401,
            "Rejects a PUT from a trusted oauth client w/o a given user")

    def test_create_trusted_client(self):
        """ Tests to be sure the create_consumer
            command can create trusted clients.
        """
        consumer_name = 'This is for testing create_consumer'

        call_command('create_consumer', consumer_name=consumer_name,
                     trusted='yes', silent=True)

        consumer = Consumer.objects.get(name=consumer_name)

        client = TrustedOAuthClient.objects.get(consumer=consumer)

        self.assertIsInstance(client, TrustedOAuthClient)
